#######################################################
# reorderGCT.py
# Author: Kevin Ying
# Date:  14 April 2011 

# Sort the rows of a GCT file by specified property 
########################################################
########################################################
# @TODO 

########################################################

########################################################
# Version History
# 1:
#	- Sort by GeneSymbol
#	  Assumes the GeneSymbol is at the start of the Description
#  	  field of the GCT
# 2:
#	- ascending descending, symbol to lowercase 
# fiorano 6.2 
# 6 July 2011
#	- Sort by avg expr level (mean)
#	- variance, sum
########################################################

# command line:
# python reorderGCT.py -g <gct.file> -o <output.file> -s <sort on feature> -a

# GenePattern command line:
# python <libdir>reorderGCT.py <gct.file> <output.file> <sort.feature> <order.as>


########################################################
# Imports
import sys
import os
from optparse import OptionParser

from common_pwbc import *

########################################################


########################################################
# MAIN
########################################################
def main():
	usage =  "Usage: %s -g <gct file> -o <output file>" % os.path.basename(sys.argv[0])
	parser = OptionParser(usage=usage)
	parser.add_option("-g", "--gctFile",
			  action="store", type="string", dest="gctFile",
			  help="The GCT file to be reordered")
	parser.add_option("-o", "--outFile",
			  action="store", type="string", dest="outFile",
			  help="The name of the output file")
	parser.add_option("-s", "--sortFeature",
			  action="store", type="string", dest="sortFeature",
			  help="The feature to sort on")
	parser.add_option("-a", "--asceDesc",
			  action="store", type="string", dest="asceDesc",
			  help="Ascending or descending sort")
	(options, args) = parser.parse_args()

	gctFile  = options.gctFile
	outFile  = options.outFile
	sortFeature = options.sortFeature
	asceDesc = options.asceDesc # acceptable values: 'a' or 'd'


	#########################################
	# Store GCT file 
	#########################################
	origGCT = readGCT(gctFile)

	#########################################
	# Sort the GCT 
	#########################################
	reverse = False 
	if (asceDesc == "d"): 
		reverse = True 

	if (sortFeature == "geneSymbol"):
		sortedData = sorted(origGCT.gctData, key=lambda row: getGeneSymbol(row).lower(), reverse=reverse)
	elif (sortFeature == "avgExprLevel"):
		sortedData = sorted(origGCT.gctData, key=lambda row: getAvgExprLevel(row), reverse=reverse)
	elif (sortFeature == "variance"):
		sortedData = sorted(origGCT.gctData, key=lambda row: getExprVariance(row), reverse=reverse)
	elif (sortFeature == "sum"):
		sortedData = sorted(origGCT.gctData, key=lambda row: getExprSum(row), reverse=reverse)

	sortedGCT = GCTFile(origGCT.header, sortedData)

	#########################################
	# Print 
	#########################################
	sortedGCT.printGCT(outFile)



########################################################

########################################################
# run main 
########################################################
if __name__ == "__main__":
	main()
